@@ -22,7 +22,7 @@
 %                                                                             %
 %                      Copyright 2017-2018 YANDEX LLC.                        %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -124,9 +124,6 @@ static MagickBooleanType IsHeifSuccess(struct heif_error *error, Image *image)
 static Image *ReadHEICImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-  const char
-    *option;
-
   const StringInfo
     *profile;
 
@@ -143,6 +140,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
     stride_cr;
 
   MagickBooleanType
+    preserve_orientation,
     status;
 
   size_t
@@ -166,7 +164,7 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   struct heif_image_handle
     *image_handle;
 
-  uint8_t
+  const uint8_t
     *p_y,
     *p_cb,
     *p_cr;
@@ -205,7 +203,6 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   heif_context=heif_context_alloc();
   error=heif_context_read_from_memory_without_copy(heif_context,file_data,
     length,NULL);
-  file_data=RelinquishMagickMemory(file_data);
   if (IsHeifSuccess(&error,image) == MagickFalse)
     {
       heif_context_free(heif_context);
@@ -311,6 +308,10 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   image->depth=8;
   image->columns=(size_t) heif_image_handle_get_width(image_handle);
   image->rows=(size_t) heif_image_handle_get_height(image_handle);
+  preserve_orientation=IsStringTrue(GetImageOption(image_info,
+    "heic:preserve-orientation"));
+  if (preserve_orientation == MagickFalse)
+    (void) SetImageProperty(image,"exif:Orientation","1");
   if (image_info->ping != MagickFalse)
     {
       image->colorspace=YCbCrColorspace;
@@ -332,16 +333,15 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   */
   (void) SetImageColorspace(image,YCbCrColorspace);
   decode_options=(struct heif_decoding_options *) NULL;
-  option=GetImageOption(image_info,"heic:preserve-orientation");
-  if (IsStringTrue(option) == MagickTrue)
+  if (preserve_orientation == MagickTrue)
     {
       decode_options=heif_decoding_options_alloc();
       decode_options->ignore_transformations=1;
     }
-  else
-    (void) SetImageProperty(image,"exif:Orientation","1");
   error=heif_decode_image(image_handle,&heif_image,heif_colorspace_YCbCr,
-    heif_chroma_420,NULL);
+    heif_chroma_420,decode_options);
+  if (decode_options != (struct heif_decoding_options *) NULL)
+    heif_decoding_options_free(decode_options);
   if (IsHeifSuccess(&error,image) == MagickFalse)
     {
       heif_image_handle_release(image_handle);
@@ -349,27 +349,23 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
       file_data=RelinquishMagickMemory(file_data);
       return(DestroyImageList(image));
     }
-  if (decode_options != (struct heif_decoding_options *) NULL)
+  /*
+    Correct the width and height of the image.
+  */
+  image->columns=(size_t) heif_image_get_width(heif_image,heif_channel_Y);
+  image->rows=(size_t) heif_image_get_height(heif_image,heif_channel_Y);
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
     {
-      /*
-        Correct the width and height of the image.
-      */
-      image->columns=(size_t) heif_image_get_width(heif_image,heif_channel_Y);
-      image->rows=(size_t) heif_image_get_height(heif_image,heif_channel_Y);
-      status=SetImageExtent(image,image->columns,image->rows);
-      heif_decoding_options_free(decode_options);
-      if (status == MagickFalse)
-        {
-          heif_image_release(heif_image);
-          heif_image_handle_release(image_handle);
-          heif_context_free(heif_context);
-          file_data=RelinquishMagickMemory(file_data);
-          return(DestroyImageList(image));
-        }
+      heif_image_release(heif_image);
+      heif_image_handle_release(image_handle);
+      heif_context_free(heif_context);
+      file_data=RelinquishMagickMemory(file_data);
+      return(DestroyImageList(image));
     }
-  p_y=heif_image_get_plane(heif_image,heif_channel_Y,&stride_y);
-  p_cb=heif_image_get_plane(heif_image,heif_channel_Cb,&stride_cb);
-  p_cr=heif_image_get_plane(heif_image,heif_channel_Cr,&stride_cr);
+  p_y=heif_image_get_plane_readonly(heif_image,heif_channel_Y,&stride_y);
+  p_cb=heif_image_get_plane_readonly(heif_image,heif_channel_Cb,&stride_cb);
+  p_cr=heif_image_get_plane_readonly(heif_image,heif_channel_Cr,&stride_cr);
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     PixelPacket
@@ -479,7 +475,7 @@ ModuleExport size_t RegisterHEICImage(void)
 #if defined(LIBHEIF_VERSION)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
-  entry->magick_module=ConstantString("GIF");
+  entry->magick_module=ConstantString("HEIC");
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
